from typing import AsyncGenerator, AsyncIterable

from yapapi import Golem, Task, WorkContext
from yapapi.payload import vm

from golocity.helpers import log


class YagnaClient:
    def __init__(self, vm_hash: str, command: list[str], budget: float, subnet: str):
        self.vm_hash = vm_hash
        self.command = command
        self.budget = budget
        self.subnet = subnet

    async def worker(
        self, context: WorkContext, tasks: AsyncIterable[Task]
    ) -> AsyncGenerator:
        async for task in tasks:
            command = " ".join(self.command)
            context.run("/bin/bash", "-c", command)

            future_results = yield context.commit()
            results = await future_results
            task.accept_result(result=results[-1])

    async def main(self) -> None:
        package = await vm.repo(
            image_hash=self.vm_hash,
        )

        tasks: list[Task] = [Task(data=None)]

        async with Golem(budget=self.budget, subnet_tag=self.subnet) as golem:
            async for completed in golem.execute_tasks(  # type: ignore[attr-defined]
                self.worker, tasks, payload=package
            ):
                log("WARNING", "Yagna: {}".format(completed.result.stdout))



import pkg_resources
import typer


TMP_DOCKERFILE: str = "Dockerfile.golocity"
VERSION: str = pkg_resources.get_distribution("golocity").version
CONFIGURATION_FILE: str = "golocity.ini"
# Holds config file and path information from main callback.
CONF: dict = {}

PROJECT_ART: str = typer.echo(
    """
      ____       _            _ _
     / ___| ___ | | ___   ___(_) |_ _   _
    | |  _ / _ \| |/ _ \ / __| | __| | | |
    | |_| | (_) | | (_) | (__| | |_| |_| |
     \____|\___/|_|\___/ \___|_|\__|\__, |
                                    |___/
            """
)



import configparser
import logging
from pathlib import Path

import typer

from golocity import CONFIGURATION_FILE


def init_config(project_path: Path) -> configparser.ConfigParser:
    config = configparser.ConfigParser()
    file_path = project_path.joinpath(CONFIGURATION_FILE)
    if file_path.exists():
        config.read(file_path)
    else:
        config["DEFAULT"] = {}
        config["DEFAULT"]["image_id"] = ""
        config["DEFAULT"]["command"] = ""
        config["DEFAULT"]["gvmkit_hash"] = ""
    return config


def init_logging(level: str) -> None:
    numeric_level = getattr(logging, level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError("Invalid log level: {}".format(level))
    logging.basicConfig(
        filename=".golocity/golocity.log",
        filemode="w",
        level=numeric_level,
        format="%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
    )


def log(level: str, msg: str) -> None:
    logging.getLogger(__name__)
    colors: dict[str, str] = {
        "DEBUG": typer.colors.CYAN,
        "INFO": typer.colors.GREEN,
        "WARNING": typer.colors.YELLOW,
        "CRITICAL": typer.colors.RED,
    }
    numeric_level = getattr(logging, level.upper(), None)
    typer.echo(
        typer.style(
            msg,
            fg=colors[level],
        )
    )
    logging.log(level=numeric_level, msg=msg)



import ast
import os
import subprocess
import sys
from pathlib import Path
from typing import BinaryIO, Iterator, Optional, TextIO

import docker

from golocity import TMP_DOCKERFILE
from golocity.helpers import log


class ImageClient:
    """
    Handles all image manipulations made by Golocity.
    """

    def __init__(
        self, project_path: Path, dockerfile_path: Path, image_id: Optional[str]
    ) -> None:
        self.docker_client: docker.DockerClient = self.setup_docker()
        self.path = project_path
        self.golocity_path: Path = self.path.joinpath(".golocity")
        self.tmp_file: Path = self.path.joinpath(TMP_DOCKERFILE)
        self.command: list[str] = self.clean_dockerfile(dockerfile=dockerfile_path)
        self.id = image_id

    @staticmethod
    def setup_docker() -> Optional[docker.DockerClient]:
        """
        Initiates docker client and checks to make sure docker engine is running.
        :return: functional docker client
        """
        try:
            client = docker.from_env()
            return client
        except docker.errors.DockerException:
            log(
                "CRITICAL",
                "Cannot connect to the Docker daemon. Is the docker daemon running?\n",
            )
            sys.exit(1)

    def clean_dockerfile(self, dockerfile: Path) -> list[str]:
        """
        Parses Dockerfile for CMD and ENTRYPOINT commands and removes them
        in preparation for GVMKit build.
        :param dockerfile: A path leading to a Dockerfile.
        :return: The stripped commands from the Dockerfile.
        """
        open_dockerfile: TextIO = open(dockerfile)
        tmp_dockerfile: BinaryIO = open(self.tmp_file, "w+b")
        command: Optional[list[str]] = []

        for line in open_dockerfile:
            if line[:3] == "CMD":
                command: list[str] = ast.literal_eval(  # type: ignore[no-redef]
                    line[3:].strip(" ")
                )
                continue
            elif line[:10] == "ENTRYPOINT":
                command: list[str] = ast.literal_eval(  # type: ignore[no-redef]
                    line[10:].strip(" ")
                )
                continue
            tmp_dockerfile.write(line.encode())
        tmp_dockerfile.close()

        if not command:
            log("WARNING", "Cannot find ENTRYPOINT or CMD specified in Dockerfile.\n")
            sys.exit(1)

        return command

    def _build(self) -> tuple[docker.DockerClient.images, Iterator[str]]:
        image, logs = self.docker_client.images.build(
            path=str(self.path), dockerfile=self.tmp_file.name, rm=True
        )
        return image, logs

    def build_image(
        self, keep_tmp: Optional[bool] = False
    ) -> tuple[docker.DockerClient.images, Iterator[str]]:
        """
        Builds image from Dockerfile
        :param keep_tmp: Whether to keep the temporary Dockerfile or not.
        :return: A tuple containing the new Image object and a generator of
                 the build logs as JSON-decoded objects.
        """
        log("INFO", "Building image from Dockerfile at {}".format(self.path))
        image, logs = self._build()
        for chunk in logs:
            if "stream" in chunk:
                for line in chunk["stream"].splitlines():  # type: ignore[index]
                    log("DEBUG", "Docker: {}".format(line))

        if not keep_tmp:
            os.remove(self.tmp_file)
        self.id = image.id

        return image, logs

    def _get(self) -> docker.DockerClient.images:
        return self.docker_client.images.get(self.id)

    def get_image(self) -> docker.DockerClient.images:
        log("INFO", "Retrieving pre-built image {}".format(self.id))
        try:
            image = self._get()
        except docker.errors.ImageNotFound:
            log(
                "CRITICAL", "The image was not found, please rebuild the project again."
            )
            sys.exit(1)
        except Exception as e:
            log("CRITICAL", "Docker: {}".format(e))
            sys.exit(1)
        return image

    def _run_process(self, arguments: list[str]) -> Optional[str]:
        process = subprocess.Popen(
            arguments, stdout=subprocess.PIPE, cwd=self.golocity_path
        )
        for stdout_line in process.stdout:  # type: ignore[union-attr]
            dl: str = stdout_line.decode("utf-8")
            log("DEBUG", "gvmkit-build: {}".format(" ".join(dl.split())))
            if "success." in dl.split(" "):
                return dl.split(" ")[3]
        return None

    def build_gvmkit(
        self,
        info: Optional[bool] = False,
        push: Optional[bool] = True,
    ) -> Optional[str]:
        """
        Builds image in Golem format and pushes new image to Golem repo.
        :param info: Whether to preform a dry run or not.
        :param push: Whether to push the image or not.
        :return: The hash of the pushed image.
        """
        # Currently, gvmkit_build is optimised for command line use only.
        # While this is suboptimal, it is the best way possible for now.
        log("INFO", "Building Golem virtual machine from image {}".format(self.id))
        args: list[str] = [
            "python",
            "-m",
            "gvmkit_build",
            self.id,
        ]  # type: ignore[list-item]
        if info:
            args.append("--info")
        try:
            if not push:
                return self._run_process(args)
            else:
                # gvmkit_build will not push an image after building,
                # we have to run the command again.
                self._run_process(args)
                args.append("--push")
                return self._run_process(args)
        except Exception as e:
            log("CRITICAL", str(e))
            sys.exit(1)



from pathlib import Path

import typer

from golocity import CONF, PROJECT_ART, VERSION
from golocity.helpers import init_config, init_logging, log


def main(
    path: Path = typer.Option(
        default=Path("."),
        help="The path to the project.",
    ),
    level: str = typer.Option(
        default="DEBUG",
        help="The level to set the logger.",
    ),
    art: bool = typer.Option(
        default=True,
        help="Whether to display terminal art or not.",
        is_flag=True,
    ),
) -> None:
    """
    Main entrypoint for Golocity
    :param path: The path to the project.
    :param level: The level to set the logger.
    :param art: Whether to display terminal art or not.
    :return: None
    """
    if art:
        PROJECT_ART

    golocity_path: Path = path.joinpath(".golocity")
    golocity_path.mkdir(exist_ok=True)
    init_logging(level=level)
    log("INFO", "Started Golocity, version {}".format(VERSION))
    CONF["path"] = path
    CONF["config"] = init_config(project_path=golocity_path)


def create_golocity() -> typer.Typer:
    from golocity.commands import build, deploy

    app: typer.Typer = typer.Typer()

    app.command(
        help="Prepares and builds image, saves pertinent information"
        "in the project configuration file."
    )(build.build)

    app.command(help="Deploys project to the Golem Network.")(deploy.deploy)

    app.callback(
        help="Golocity version "
        + typer.style("{}".format(VERSION), fg=typer.colors.MAGENTA)
    )(main)

    return app


golocity: typer.Typer = create_golocity()



import asyncio
import json
import sys
from configparser import ConfigParser
from pathlib import Path

import typer
from yapapi.log import enable_default_logger

from golocity import CONF
from golocity.helpers import log
from golocity.yagna import YagnaClient


def deploy(
    budget: float = typer.Argument(
        default=None,
        help="The limit to spend on deploying the project.",
    ),
    subnet: str = typer.Option(
        default="mainnet",
        help="Which subnet to deploy the project to.",
    ),
) -> None:
    """
    Deploys project to the Golem Network.
    :param budget: The limit to spend on deploying the project.
    :param subnet: Which subnet to deploy the project to.
    :return: None
    """
    path: Path = CONF["path"]
    config: ConfigParser = CONF["config"]
    golocity_path: Path = path.joinpath(".golocity")

    log("INFO", "Deploying project to Golem Network.")

    if golocity_path.is_dir():
        try:
            enable_default_logger()
            ya_client = YagnaClient(
                vm_hash=config["DEFAULT"]["gvmkit_hash"],
                command=json.loads(config["DEFAULT"]["command"]),
                budget=budget,
                subnet=subnet,
            )
            loop = asyncio.get_event_loop()
            task = loop.create_task(ya_client.main())
            loop.run_until_complete(task)
        except KeyError:
            log(
                "CRITICAL",
                "Golocity does not have all the parameters needed"
                "to deploy this project.\n"
                "Try building the project and try again.",
            )
            sys.exit(1)
        except Exception as e:
            log("CRITICAL", str(e))
            sys.exit(1)



import json
from configparser import ConfigParser
from pathlib import Path
from typing import Iterator

import docker
import typer

from golocity import CONF, CONFIGURATION_FILE
from golocity.helpers import log
from golocity.image import ImageClient


def build(
    dockerfile: str = typer.Option(
        default="Dockerfile",
        help="The name of the Dockerfile for the project.",
    ),
    info: bool = typer.Option(
        default=False,
        help="Preform a test build without creating image "
        "and pushing image to the Golem repository.",
        is_flag=True,
    ),
    push: bool = typer.Option(
        default=True,
        help="Push created image to Golem repository.",
        is_flag=True,
    ),
    keep_docker_image: bool = typer.Option(
        default=False,
        help="Keep the docker image if already built.",
        is_flag=True,
        metavar="keep-docker-image",
    ),
    keep_tmp: bool = typer.Option(
        default=False,
        help="Keep the temporary Dockerfile used by Golocity.",
        is_flag=True,
        metavar="keep-tmp",
    ),
) -> None:
    """
    Prepares and builds image, saves pertinent information in the
    project config file.
    :param dockerfile: The name of the Dockerfile for the project.
    :param info: Preform a test build without creating image and
                 pushing image to the Golem repository.
    :param push: Push created image to Golem repository.
    :param keep_docker_image: Keep the docker image if already built.
    :param keep_tmp: Keep the temporary Dockerfile used by Golocity.
    :return: None
    """
    path: Path = CONF["path"]
    config: ConfigParser = CONF["config"]
    golocity_path: Path = path.joinpath(".golocity")
    golocity_path.mkdir(exist_ok=True)

    dockerfile_path: Path = path.joinpath(dockerfile)
    image_client: ImageClient = ImageClient(
        project_path=path,
        dockerfile_path=dockerfile_path,
        image_id=config["DEFAULT"]["image_id"],
    )
    if keep_docker_image:
        image: docker.DockerClient.images = image_client.get_image()
    else:
        converted_image: docker.DockerClient.images
        logs: Iterator[str]
        image, logs = image_client.build_image(keep_tmp=keep_tmp)

    config["DEFAULT"]["command"] = json.dumps(image_client.command)
    vm_hash = image_client.build_gvmkit(info=info, push=push)

    config["DEFAULT"]["image_id"] = image.id
    if vm_hash is not None:
        config["DEFAULT"]["gvmkit_hash"] = vm_hash

    with open(golocity_path.joinpath(CONFIGURATION_FILE), "w") as config_file:
        config.write(config_file)

    log("INFO", "Successfully built project 🎉")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['golocity', 'golocity.commands']

package_data = \
{'': ['*']}

install_requires = \
['gvmkit-build>=0.2.6,<0.3.0',
 'typer[all]>=0.3.2,<0.4.0',
 'yapapi>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['golocity = golocity.main:golocity']}

setup_kwargs = {
    'name': 'golocity',
    'version': '0.0.1',
    'description': 'An easy to use execution manager for the Golem Network',
    'long_description': "========\nGolocity\n========\n\nSummary\n=======\n\n    Golocity is a young project and while there is testing,\n    there still may be bugs. Be sure and test thoroughly prior to use in a production environment.\n\n**Golocity** is an easy to use CLI execution manager for the Golem Network.\nIt aims to convert and deploy your dockerized project on the Golem Network in\nas little time as possible. It is perfect for developers who want to deploy\ntheir preexisting projects on the Network or those who need a starting point\nto harness all of the Network's features.\n\nInstallation\n============\n\nGolocity is available on ``PyPI``. This will install the latest stable version.\n\nTo install Golocity, simply use ``pip``:\n\n.. code-block:: console\n\n    pip install --user golocity\n\n\nGetting Started\n===============\n\nBuilding the images\n-------------------\n\nBefore your project can be deployed on the Network, we must first build the docker\nimage as well as the Golem virtual machine image. Before issuing the command, make\nsure you have Docker installed and running on your computer.\n\nTo do so, use the ``build`` command:\n\n.. code-block:: console\n\n    golocity build /path/to/your/project\n\nThis will build the requisite images as well as create a ``.golocity``\ndirectory in your projects directory. This directory holds configuration files needed\nby Golocity as well as logs.\n\nThe build command also pushes the Golem virtual machine image to the Network's public\nrepository. To preform a dry-run, append the ``--info`` flag to the command.\n\nUnder the hood, Golocity parses your Dockerfile for ``ENTRYPOINT`` and ``CMD`` commands.\nThese commands currently are not supported in the Golem virtual machine format, so\nGolocity removes them and manually calls them from within the ``deploy`` command. Don't\nworry, Golocity will not alter your project files, it only operates on temporary copies.\n\nDeploying to the Network\n------------------------\n\nOnce you have successfully built and pushed the images, you are ready to deploy your\nproject on the Network! First, make sure you have the ``yagna`` daemon running on your\nmachine, for more information, refer to the `Golem Handbook.\n<https://handbook.golem.network/requestor-tutorials/flash-tutorial-of-requestor-development/run-first-task-on-golem>`_\n\nNow, deploying is one line away!\n\n.. code-block:: console\n\n    golocity deploy [budget]\n\nReplace ``budget`` with the limit of what to spend while running the project. From here\nGolocity will handle the rest. It will find the best provider and handle the output.\n\nContributing\n=============\n\nPoetry\n------\n\nWe use `poetry <https://github.com/sdispater/poetry>`_ to manage dependencies, to\nget started follow these steps:\n\n.. code-block:: console\n\n    git clone https://github.com/davidstyers/golocity\n    cd golocity\n    poetry install\n    poetry run pytest\n\nPre-Commit\n----------\n\nWe have a configuration for\n`pre-commit <https://github.com/pre-commit/pre-commit>`_ to add the hook run the\nfollowing command:\n\n.. code-block:: console\n\n    pre-commit install\n",
    'author': 'davidstyers',
    'author_email': 'david@styers.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/davidstyers/golocity',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
